#!/usr/local/bin/python

from shaders import *
from GLTools import *
from utils import *
from Application import *

CANVAS_QUALITY = 2 # 2 is the max

class Brush:

    NORMAL=1
    INCREMENTAL=3
    RANDOM=2

    #shared properties
    position = vec([0,0])
    color = [1.0, 1.0, 1.0]
    size = [100,100]
    preassure = 0.1
    time_interval_in_ms = 50 #in secs
    angle = 0
    brush_texture_mode = INCREMENTAL
    brush_rotation_mode = RANDOM
    brush_size_mode = NORMAL
    additive_blend = True
    
    last_texture_used = 0
    last_draw_time = 0

    def __init__(self,textures):
        self.textures = []
        
        if len(textures) > 0:
            self.addTextures(textures)

    def addTextures(self,filename_list):

        for name in filename_list:
            t = Texture()
            t.load(name)
            self.textures.append(t)

    def render(self, final = True):
        if final and self.time_interval_in_ms + self.last_draw_time > Application.instance.current_time:
            return #nothing to draw

        if self.brush_texture_mode == Brush.RANDOM:
            t = random.choice( self.textures )
        elif self.brush_texture_mode == Brush.INCREMENTAL:
            t = self.textures[ self.last_texture_used ]
        else:
             t = self.textures[0]
             
        t.bind()
        glColor4f(self.color[0],self.color[1],self.color[2],self.preassure if final else self.preassure * 2)

        if not self.additive_blend:
            glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
        else:
            glBlendFunc(GL_SRC_ALPHA,GL_ONE)
        glEnable(GL_BLEND)        

        glBegin( GL_QUADS )
        glTexCoord2f(0,0), glVertex3f( -self.size[0], -self.size[0], 0)
        glTexCoord2f(1,0), glVertex3f( +self.size[0], -self.size[0], 0)
        glTexCoord2f(1,1), glVertex3f( +self.size[0], +self.size[1], 0)
        glTexCoord2f(0,1), glVertex3f( -self.size[0], +self.size[1], 0)
        glEnd()

        if final:
            self.update()

    def renderInPos(self):
        glPushMatrix()
        glTranslate(self.position[0],self.position[1],0.0)
        glRotatef(self.angle,0.0,0.0,1.0)
        self.render(False)
        glPopMatrix()

    def paint(self, canvas):
        canvas.rt.enable()
        enableRenderHUD(0,canvas.size[0], 0,canvas.size[1])

        glMatrixMode( GL_MODELVIEW )
        glPushMatrix()
        glTranslatef( self.position[0] - canvas.offset[0], self.position[1] - canvas.offset[1], 0.0 )
        glRotatef(self.angle,0.0,0.0,1.0)
        self.render(True)
        glPopMatrix()

        disableRenderHUD()
        canvas.rt.disable()

    def update(self):
        self.last_draw_time = Application.instance.current_time

        if self.brush_texture_mode == Brush.INCREMENTAL:
            self.incTexture()
        elif self.brush_texture_mode == Brush.RANDOM:
            self.last_texture_used = random.randint( len(self.textures))

        if self.brush_rotation_mode == Brush.INCREMENTAL:
            self.angle += 5
        elif self.brush_rotation_mode == Brush.RANDOM:
            self.angle += random.randint(0,360)
        if self.brush_size_mode == Brush.RANDOM:
            self.size = [ random.randint(0,100),random.randint(0,100) ]

    def incTexture(self):
        self.last_texture_used += 1
        if self.last_texture_used >= len(self.textures):
            self.last_texture_used = 0

    def setPosition(self,pos):
        self.position = vec(pos)

class Canvas:
    bgcolor = [0.0,0.0,0.0]
    quality = [[GL_RGB,GL_UNSIGNED_BYTE],[GL_RGB16F_ARB,GL_UNSIGNED_SHORT],[GL_RGB32F_ARB,GL_FLOAT] ]
    render_grid = True
    
    def __init__(self,size):
        global CANVAS_QUALITY
        self.offset = [0,0]

        self.rt = RenderTexture(size,Canvas.quality[ CANVAS_QUALITY ][0],{}, Canvas.quality[ CANVAS_QUALITY ][1]) #GL_RGB32F_ARB,{},GL_FLOAT
        self.size = size
        self.clear()

        self.createUndo()

    def clear(self):
        enableRenderHUD(0,self.size[0], self.size[1], 0)
        self.rt.enable()        
        glClearColor(self.bgcolor[0], self.bgcolor[1], self.bgcolor[2], 1.0)
        glClear(GL_COLOR_BUFFER_BIT)
        self.rt.disable()
        disableRenderHUD()        

    def render(self, window_size):
        
        glMatrixMode( GL_MODELVIEW )
        glPushMatrix()
        glTranslate( self.offset[0], self.offset[1], 0.0)
        self.rt.render([0,0],self.size)

        #frame
        if Canvas.render_grid:
            glBlendFunc(GL_SRC_ALPHA,GL_ONE)
            glEnable(GL_BLEND)            
            glDisable(GL_TEXTURE_2D)
            glLineWidth(2)
            glColor4f(0.25,0.25,0.25,0.5)
            glBegin(GL_LINE_LOOP)
            glVertex2f(0,0),glVertex2f(self.size[0],0),glVertex2f(self.size[0],self.size[1]),glVertex2f(0,self.size[1])
            glEnd()
        glPopMatrix()

    def createUndo(self):
        self.undo_rt = RenderTexture(self.rt.size,self.rt.format,{},self.rt.format_type)
        self.saveUndo()
 
    def saveUndo(self):
        if not hasattr(self,"undo_rt"):
            return
        self.undo_rt.enable()
        
        enableRenderHUD(0,self.size[0], 0,self.size[1])
        glMatrixMode( GL_MODELVIEW )
        glPushMatrix()
        #glTranslate( self.offset[0], self.offset[1], 0.0)
        self.rt.render([0,0],self.size)
        glPopMatrix()        
        self.undo_rt.disable()
        disableRenderHUD()

    def recoverUndo(self):
        if not hasattr(self,"undo_rt"):
            return
        self.rt,self.undo_rt = self.undo_rt,self.rt

    def increaseOffset(self,delta):
        self.offset = [ self.offset[0] + delta[0], self.offset[1] + delta[1]]

    def update(self):
        #self.rt.fill([0.1,0.1,0.1,0.01])
        pass
